#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
"""

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2021 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import abc
from bisect import bisect_left
from collections import OrderedDict
import functools
import operator
from time import perf_counter, time

from typing import Type

from pyxcp.transport.base import BaseTransport
from pyxcp.config import Configuration
from pyxcp.types import NumericType


CAN_EXTENDED_ID         = 0x80000000
MAX_11_BIT_IDENTIFIER   = (1 << 11) - 1
MAX_29_BIT_IDENTIFIER   = (1 << 29) - 1
MAX_DLC_CLASSIC         = 8
CAN_FD_DLCS             = (12, 16, 20, 24, 32, 48, 64)  # Discrete CAN-FD DLCs in case DLC > 8.


class IdentifierOutOfRangeError(Exception):
    """Signals an identifier greater then :obj:`MAX_11_BIT_IDENTIFIER` or :obj:`MAX_29_BIT_IDENTIFIER`."""

    pass


def isExtendedIdentifier(identifier: int) -> bool:
    """Check for extendend CAN identifier.

    Parameters
    ----------
    identifier: int

    Returns
    -------
    bool
    """
    return (identifier & CAN_EXTENDED_ID) == CAN_EXTENDED_ID


def stripIdentifier(identifier: int) -> int:
    """Get raw CAN identifier (remove :obj:`CAN_EXTENDED_ID` bit if present).

    Parameters
    ----------
    identifier: int

    Returns
    -------
    int
    """
    return identifier & (~CAN_EXTENDED_ID)


def samplePointToTsegs(tqs: int, samplePoint: float) -> tuple:
    """Calculate TSEG1 and TSEG2 from time-quantas and sample-point.

    Parameters
    ----------
    tqs: int
        Number of time-quantas
    samplePoint: float or int
        Sample-point as a percentage value.

    Returns
    -------
    tuple (TSEG1, TSEG2)
    """
    factor = samplePoint / 100.0
    tseg1 = int(tqs * factor)
    tseg2 = tqs - tseg1
    return (tseg1, tseg2)


def padFrame(frame: bytes, padding_value: int, padding_len:int = 0) -> bytes:
    """Pad frame to next discrete DLC value.

    References:
    -----------
    ISO/DIS 15765 - 4; 8.2 Data length Code (DLC)
    AUTOSAR CP Release 4.3.0, Specification of CAN Transport Layer; 7.3.8 N-PDU padding
    AUTOSAR CP Release 4.3.0, Specification of CAN Driver; [SWS_CAN_00502], [ECUC_Can_00485]
    AUTOSAR CP Release 4.3.0, Requirements on CAN; [SRS_Can_01073], [SRS_Can_01086], [SRS_Can_01160]
    """
    frame_len = max(len(frame), padding_len)
    if frame_len <= MAX_DLC_CLASSIC:
        actual_len = MAX_DLC_CLASSIC
    else:
        actual_len = CAN_FD_DLCS[bisect_left(CAN_FD_DLCS, frame_len)]
    # append fill bytes up to MAX_DLC resp. next discrete FD DLC.
    if len(frame) < actual_len:
        frame += bytes([padding_value]) * (actual_len - len(frame))
    return frame

class Identifier:
    """Convenience class for XCP formatted CAN identifiers.

    Parameters:
    -----------
    raw_id: int
        Bit 32 set (i.e. 0x80000000) signals an extended (29-bit) identifier.

    Raises
    ------
    :class:`IdentifierOutOfRangeError`
    """

    def __init__(self, raw_id: int):
        self._raw_id = raw_id
        self._id = stripIdentifier(raw_id)
        self._is_extended = isExtendedIdentifier(raw_id)
        if self._is_extended:
            if self._id > MAX_29_BIT_IDENTIFIER:
                raise IdentifierOutOfRangeError("29-bit identifier '{}' is out of range".format(self._id))
        else:
            if self._id > MAX_11_BIT_IDENTIFIER:
                raise IdentifierOutOfRangeError("11-bit identifier '{}' is out of range".format(self._id))

    @property
    def id(self) -> int:
        """
        Returns
        -------
        int
            Identifier as seen on bus.
        """
        return self._id

    @property
    def raw_id(self) -> int:
        """
        Returns
        -------
        int
            Raw XCP formatted identifier.
        """
        return self._raw_id

    @property
    def is_extended(self) -> bool:
        """
        Returns
        -------
        bool
            - True - 29-bit identifier.
            - False - 11-bit identifier.
        """
        return self._is_extended

    @staticmethod
    def make_identifier(identifier: int, extended: bool) -> int:
        """Factory method.

        Parameters
        ----------
        identifier: int
            Identifier as seen on bus.

        extended: bool
            bool
                - True - 29-bit identifier.
                - False - 11-bit identifier.
        Returns
        -------
        :class:`Identifier`

        Raises
        ------
        :class:`IdentifierOutOfRangeError`
        """
        return Identifier(identifier if not extended else (identifier | CAN_EXTENDED_ID))

    def __eq__(self, other):
        return (self.id == other.id) and (self.is_extended == other.is_extended)

    def __str__(self):
        return "Identifier(id = 0x{:08x}, is_extended = {})".format(self.id, self.is_extended)

    def __repr__(self):
        return "Identifier(0x{:08x})".format(self.raw_id)


class Frame:
    """"""

    def __init__(self, id_: Identifier, dlc: int, data: bytes, timestamp: int):
        self.id = id_
        self.dlc = dlc
        self.data = data
        self.timestamp = timestamp

    def __repr__(self):
        return "Frame(id = 0x{:08x}, dlc = {}, data = {}, timestamp = {})".format(
            self.id, self.dlc, self.data, self.timestamp
        )

    __str__ = __repr__


class CanInterfaceBase(metaclass=abc.ABCMeta):
    """
    Abstract CAN interface handler that can be implemented for any actual CAN device driver
    """

    PARAMETER_MAP = {}

    @abc.abstractmethod
    def init(self, parent, receive_callback):
        """
        Must implement any required action for initing the can interface

        Parameters
        ----------
        parent: :class:`Can`
            Refers to owner.
        receive_callback: callable
            Receive callback function to register with the following argument: payload: bytes
        """

    @abc.abstractmethod
    def transmit(self, payload: bytes):
        """
        Must transmit the given payload on the master can id.

        Parameters
        ----------
        payload: bytes
            payload to transmit
        """

    @abc.abstractmethod
    def close(self):
        """ Must implement any required action for disconnecting from the can interface """

    @abc.abstractmethod
    def connect(self):
        """Open connection to can interface"""

    @abc.abstractmethod
    def read(self):
        """Read incoming data"""

    @abc.abstractmethod
    def getTimestampResolution(self):
        """Get timestamp resolution in nano seconds."""

    def loadConfig(self, config):
        """Load configuration data."""
        self.config = Configuration(self.PARAMETER_MAP or {}, config or {})


class EmptyHeader:
    """ There is no header for XCP on CAN  """

    def pack(self, *args, **kwargs):
        return b""


# can.detect_available_configs()


class Can(BaseTransport):
    """"""

    PARAMETER_MAP = {
        #                           Type            Req'd   Default
        "CAN_DRIVER":               (str,           True,   None),
        "CHANNEL":                  (str,           False,  ""),
        "MAX_DLC_REQUIRED":         (bool,          False,  False),
        "MAX_CAN_FD_DLC":           (int,           False,  64),
        "PADDING_VALUE":            (int,           False,   0),
        "CAN_USE_DEFAULT_LISTENER": (bool,          False,  True),
        # defaults to True, in this case the default listener thread is used.
        # If the canInterface implements a listener service, this parameter
        # can be set to False, and the default listener thread won't be started.
        "CAN_ID_MASTER":            (int,           True,   None),
        "CAN_ID_SLAVE":             (int,           True,   None),
        "CAN_ID_BROADCAST":         (int,           False,  None),
        "BITRATE":                  (int,           False,  250000),
        "RECEIVE_OWN_MESSAGES":     (bool,          False,  False),
    }

    PARAMETER_TO_KW_ARG_MAP = {
        "RECEIVE_OWN_MESSAGES": "receive_own_messages",
        "CHANNEL": "channel",
        "BITRATE": "bitrate",
    }

    MAX_DATAGRAM_SIZE = 7
    HEADER = EmptyHeader()
    HEADER_SIZE = 0

    def __init__(self, config=None):
        """init for CAN transport
        :param config: configuration
        """
        super().__init__(config)
        self.loadConfig(config)
        drivers = registered_drivers()
        interfaceName = self.config.get("CAN_DRIVER")
        if not interfaceName in drivers:
            raise ValueError(
                "{} is an invalid driver name -- choose from {}".format(interfaceName, [x for x in drivers.keys()])
            )
        canInterfaceClass = drivers[interfaceName]
        self.canInterface = canInterfaceClass()
        self.useDefaultListener = self.config.get("CAN_USE_DEFAULT_LISTENER")
        self.can_id_master = Identifier(self.config.get("CAN_ID_MASTER"))
        self.can_id_slave = Identifier(self.config.get("CAN_ID_SLAVE"))
        self.canInterface.loadConfig(config)
        self.canInterface.init(self, self.dataReceived)
        #
        # Regarding CAN-FD s. AUTOSAR CP Release 4.3.0, Requirements on CAN; [SRS_Can_01160] Padding of bytes due to discrete CAN FD DLC]:
        #   "... If a PDU does not exactly match these configurable sizes the unused bytes shall be padded."
        #
        self.max_dlc_required = self.config.get("MAX_DLC_REQUIRED") or self.canInterface.is_fd
        self.padding_value = self.config.get("PADDING_VALUE")
        self.padding_len = self.config.get("MAX_CAN_FD_DLC") if self.canInterface.is_fd else MAX_DLC_CLASSIC

    def dataReceived(self, payload: bytes, recv_timestamp: float = None):
        self.processResponse(payload, len(payload), counter=self.counterReceived + 1, recv_timestamp=recv_timestamp)

    def listen(self):
        while True:
            if self.closeEvent.isSet():
                return
            frame = self.canInterface.read()
            if frame:
                self.dataReceived(frame.data, frame.timestamp)

    def connect(self):
        if self.useDefaultListener:
            self.startListener()
        self.canInterface.connect()
        self.status = 1  # connected

    def send(self, frame):
        # XCP on CAN trailer: if required, FILL bytes must be appended
        if self.max_dlc_required:
            frame = padFrame(frame, self.padding_value, self.padding_len)
        # send the request
        if self.perf_counter_origin > 0:
            self.pre_send_timestamp = time()
            self.canInterface.transmit(payload = frame)
            self.post_send_timestamp = time()
        else:
            pre_send_timestamp = perf_counter()
            self.canInterface.transmit(payload = frame)
            post_send_timestamp = perf_counter()
            self.pre_send_timestamp = self.timestamp_origin + pre_send_timestamp - self.perf_counter_origin
            self.post_send_timestamp = self.timestamp_origin + post_send_timestamp - self.perf_counter_origin

    def closeConnection(self):
        if hasattr(self, "canInterface"):
            self.canInterface.close()

    def close(self):
        self.closeConnection()


def setDLC(length: int):
    """Return DLC value according to CAN-FD.

    :param length: Length value to be mapped to a valid CAN-FD DLC.
                   ( 0 <= length <= 64)
    """

    if length < 0:
        raise ValueError("Non-negative length value required.")
    elif length <= MAX_DLC_CLASSIC:
        return length
    elif length <= 64:
        for dlc in CAN_FD_DLCS:
            if length <= dlc:
                return dlc
    else:
        raise ValueError("DLC could be at most 64.")


def calculateFilter(ids: list):
    """
    :param ids: An iterable (usually list or tuple) containing CAN identifiers.

    :return: Calculated filter and mask.
    :rtype: tuple (int, int)
    """
    any_extended_ids = any(isExtendedIdentifier(i) for i in ids)
    raw_ids = [stripIdentifier(i) for i in ids]
    cfilter = functools.reduce(operator.and_, raw_ids)
    cmask = functools.reduce(operator.or_, raw_ids) ^ cfilter
    cmask ^= 0x1FFFFFFF if any_extended_ids else 0x7FF
    return (cfilter, cmask)


def try_to_install_system_supplied_drivers():
    """Register available pyxcp CAN drivers."""
    import importlib
    import pkgutil
    import pyxcp.transport.candriver as cdr

    for _, modname, _ in pkgutil.walk_packages(cdr.__path__, "{}.".format(cdr.__name__)):
        try:
            importlib.import_module(modname)
        except Exception as e:
            pass


def registered_drivers():
    """
    Returns
    -------
    dict (name, class)
        Dictionary containing CAN driver names and classes of all
        available drivers (pyxcp supplied and user-defined).
    """
    sub_classes = CanInterfaceBase.__subclasses__()
    return OrderedDict(zip(([c.__name__ for c in sub_classes]), sub_classes))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2018 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys

from time import time, perf_counter, get_clock_info
from binascii import hexlify


def hexDump(arr):
    if isinstance(arr, (bytes, bytearray)):
        size = len(arr)
        try:
            arr = arr.hex()
        except:
            arr = hexlify(arr).decode("ascii")
        return "[{}]".format(" ".join([arr[i * 2 : (i + 1) * 2] for i in range(size)]))
    elif isinstance(arr, (list, tuple)):
        arr = bytes(arr)
        size = len(arr)
        try:
            arr = arr.hex()
        except:
            arr = hexlify(arr).decode("ascii")
        return "[{}]".format(" ".join([arr[i * 2 : (i + 1) * 2] for i in range(size)]))
    else:
        return "[{}]".format(" ".join(["{:02x}".format(x) for x in arr]))


def slicer(iterable, sliceLength, converter=None):
    if converter is None:
        converter = type(iterable)
    length = len(iterable)
    return [converter((iterable[item : item + sliceLength])) for item in range(0, length, sliceLength)]


def flatten(*args):
    result = []
    for arg in list(args):
        if hasattr(arg, "__iter__"):
            result.extend(flatten(*arg))
        else:
            result.append(arg)
    return result


# def intToArray(value):
#    result = []
#    while value:
#        result.append(value & 0xff)
#        value >>= 8
#    if result:
#        return list(reversed(result))
#    else:
#        return [0]

# class Curry:
#    def __init__(self, fun, *args, **kwargs):
#        self.fun = fun
#        self.pending = args[:]
#        self.kwargs = kwargs.copy()
#
#    def __call__(self, *args, **kwargs):
#        if kwargs and self.kwargs:
#            kw = self.kwargs.copy()
#            kw.update(kwargs)
#        else:
#            kw = kwargs or self.kwargs
#        return self.fun(*(self.pending + args), **kw)


def getPythonVersion():
    return sys.version_info


PYTHON_VERSION = getPythonVersion()


def time_perfcounter_correlation():
    """Get the `perf_counter` value nearest to when time.time() is updated if the `time.time` on
    this platform has a resolution higher than 10us. This is tipical for the Windows platform
    were the beste resolution is ~500us.

    On non Windows platforms the current time and perf_counter is directly returned since the
    resolution is tipical ~1us.

    Note this value is based on when `time.time()` is observed to update from Python, it is not
    directly returned by the operating system.

    :return:
        (t, performance_counter) time.time value and perf_counter value when the time.time
        is updated

    """

    # use this if the resolution is higher than 10us
    if get_clock_info("time").resolution > 1e-5:
        t0 = time()
        while True:
            t1, performance_counter = time(), perf_counter()
            if t1 != t0:
                break
    else:
        return time(), perf_counter()
    return t1, performance_counter


# def createStringBuffer(*args):
#    """Create a string with file-like behaviour (StringIO on Python 2.x).
#    """
#    return StringIO(*args)


# def binExtractor(fname, offset, length):
#    """Extract a junk of data from a file.
#    """
#    fp = open(fname)
#    fp.seek(offset)
#    data = fp.read(length)
#    return data
#


#
# CYG_PREFIX = "/cygdrive/"
#
#
# def cygpathToWin(path):
#    if path.startswith(CYG_PREFIX):
#        path = path[len(CYG_PREFIX):]
#        driveLetter = "{0}:\\".format(path[0])
#        path = path[2:].replace("/", "\\")
#        path = "{0}{1}".format(driveLetter, path)
#    return path
#
#
# class StructureWithEnums(ctypes.Structure):
#    """Add missing enum feature to ctypes Structures.
#    """
#    _map = {}
#
#    def __getattribute__(self, name):
#        _map = ctypes.Structure.__getattribute__(self, '_map')
#        value = ctypes.Structure.__getattribute__(self, name)
#        if name in _map:
#            EnumClass = _map[name]
#            if isinstance(value, ctypes.Array):
#                return [EnumClass(x) for x in value]
#            else:
#                return EnumClass(value)
#        else:
#            return value
#
#    def __str__(self):
#        result = []
#        result.append("struct {0} {{".format(self.__class__.__name__))
#        for field in self._fields_:
#            attr, attrType = field
#            if attr in self._map:
#                attrType = self._map[attr]
#            value = getattr(self, attr)
#            result.append("    {0} [{1}] = {2!r};".format(
#                attr, attrType.__name__, value))
#        result.append("};")
#        return '\n'.join(result)
#
#    __repr__ = __str__
#
#
# class CommandError(Exception):
#    pass
#
#
# def runCommand(cmd):
#    proc = subprocess.Popen(
#        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
#    result = proc.communicate()
#    proc.wait()
#    if proc.returncode:
#        raise CommandError("{0}".format(result[1]))
#    return result[0]
#
#
# class SingletonBase(object):
#    _lock = threading.Lock()
#
#    def __new__(cls, *args, **kws):
#        # Double-Checked Locking
#        if not hasattr(cls, '_instance'):
#            try:
#                cls._lock.acquire()
#                if not hasattr(cls, '_instance'):
#                    cls._instance = super(SingletonBase, cls).__new__(cls)
#            finally:
#                cls._lock.release()
#        return cls._instance
#

#  class RepresentationMixIn(object):
#
#      def __repr__(self):
#          keys = [k for k in self.__dict__ if not (k.startswith('__') and k.endswith('__'))]
#          result = []
#          result.append("{0!s} {{".format(self.__class__.__name__))
#          for key in keys:
#              value = getattr(self, key)
#              if isinstance(value, (int, )):
#                  line = "    {0!s} = 0x{1:X}".format(key, value)
#              elif isinstance(value, (float, types.NoneType)):
#                  line = "    {0!s} = {1!s}".format(key, value)
#              elif isinstance(value, array):
#                  line = "    {0!s} = {1!s}".format(key, hexDump(value))
#              else:
#                  line = "    {0!s} = '{1!s}'".format(key, value)
#              result.append(line)
#          result.append("}")
#          return '\n'.join(result)
#

# def memoryMap(filename, writeable=False):
#    size = os.path.getsize(filename)
#    fd = os.open(filename, os.O_RDWR if writeable else os.O_RDONLY)
#    return mmap.mmap(
#        fd, size, access=mmap.ACCESS_WRITE if writeable else mmap.ACCESS_READ)
#
#
# if sys.platform == "win32" and WINAPI:
#
#    # Code snippet taken from
#    # http://code.activestate.com/recipes/496767-set-process-priority-in-windows/
#    # Licenced under PSF.
#
#    def setpriority(pid=None, priority=1):
#        """ Set The Priority of a Windows Process.  Priority is a value
#            between 0-5 where 2 is normal priority.  Default sets the priority
#            of the current python process but can take any valid process ID."""
#        priorityclasses = [win32process.IDLE_PRIORITY_CLASS,
#                           win32process.BELOW_NORMAL_PRIORITY_CLASS,
#                           win32process.NORMAL_PRIORITY_CLASS,
#                           win32process.ABOVE_NORMAL_PRIORITY_CLASS,
#                           win32process.HIGH_PRIORITY_CLASS,
#                           win32process.REALTIME_PRIORITY_CLASS]
#        if pid is None:
#            # pid = win32api.GetCurrentProcessId()
#            pid = ctypes.windll.kernel32.GetCurrentProcessId()
#        handle = ctypes.windll.kernel32.OpenProcess(
#            win32con.PROCESS_ALL_ACCESS, True, pid)
#        win32process.SetPriorityClass(handle, priorityclasses[priority])
#        # Chris
#        win32process.SetProcessPriorityBoost(handle, False)
#        ct = ctypes.windll.kernel32.GetCurrentThread()
#        gle = win32api.GetLastError()
#        print("gct", win32api.FormatMessage(gle))
#        ctypes.windll.kernel32.SetThreadPriority(
#            ct, win32con.THREAD_BASE_PRIORITY_LOWRT)
# else:
#    def setpriority(pid=None, priority=1):
#        pass



#!/bin/env python

import distutils
import os
import platform
import setuptools, setuptools.command.build_py, setuptools.command.develop
import subprocess
import sys

with open(os.path.join("pyxcp", "version.py"), "r") as f:
    for line in f:
        if line.startswith("__version__"):
            version = line.split("=")[-1].strip().strip('"')
            break

with open("README.md", "r") as fh:
    long_description = fh.read()

install_reqs = [
    "pyusb",
    "construct >= 2.9.0",
    "mako",
    "pyserial",
    "toml",
]
setup_reqs = [
    "pyusb",
    "construct >= 2.9.0",
    "mako",
    "pyserial",
    "toml",
]


class AsamKeyDllAutogen(distutils.cmd.Command):
    """Custom command to compile `asamkeydll.exe`."""

    description = "Compile `asamkeydll.exe`."

    def initialize_options(self):
        pass

    def finalize_options(self):
        """Post-process options."""
        asamkeydll = os.path.join("pyxcp", "asamkeydll.c")
        target = os.path.join("pyxcp", "asamkeydll.exe")
        self.arguments = [asamkeydll, "-o{}".format(target)]

    def run(self):
        """Run gcc"""
        word_width, _ = platform.architecture()
        if sys.platform == "win32" and word_width == "64bit":
            gccCmd = ["gcc", "-m32", "-O3", "-Wall"]
            self.announce(" ".join(gccCmd + self.arguments), level=distutils.log.INFO)
            try:
                subprocess.check_call(gccCmd + self.arguments)
            except Exception as e:
                print("Building pyxcp/asamkeydll.exe failed: '{}'".format(str(e)))
            else:
                print("Successfully  build pyxcp/asamkeydll.exe")


class CustomBuildPy(setuptools.command.build_py.build_py):
    def run(self):
        self.run_command("asamkeydll")
        super().run()


class CustomDevelop(setuptools.command.develop.develop):
    def run(self):
        self.run_command("asamkeydll")
        super().run()


setuptools.setup(
    name="pyxcp",
    version=version,
    provides=["pyxcp"],
    description="Universal Calibration Protocol for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Christoph Schueler",
    author_email="cpu12.gems@googlemail.com",
    url="https://github.com/christoph2/pyxcp",
    packages=setuptools.find_packages(),
    cmdclass={
        "asamkeydll": AsamKeyDllAutogen,
        "build_py": CustomBuildPy,
        "develop": CustomDevelop,
    },
    include_package_data=True,
    install_requires=install_reqs,
    setup_requires=setup_reqs,
    extras_require={"docs": ["sphinxcontrib-napoleon"], "develop": ["bumpversion"]},
    package_dir={"tests": "pyxcp/tests"},
    tests_require=["pytest", "pytest-runner"],
    test_suite="pyxcp.tests",
    license="GPLv2",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: Scientific/Engineering",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
